logmode +addin

'check that an object exists
%type = @getthistype
if %type="NONE" then
	@uiprompt("No object found, please open an equation object.")
	stop
endif

'check that _this object is an equation
if %type<>"EQUATION" then
	@uiprompt("Procedure can only be run from an equation object.")
	stop
endif

'check that equation was estimated by list. 
if _this.@bylist = 0 then
	@uiprompt("Procedure can only be run from equations estimated by list.")
	stop
endif

'error on panels
if @ispanel then
	@uiprompt("Procedure cannot be run in panel workfiles")
	stop
endif


%estcmnd = _this.@command
%coefnames = _this.@varlist
%coefnames = @wmid(%coefnames,2)  'drop dependent
%origsmpl = @pagesmpl
%origcmnd= _this.@command

'Rolling specification
!windowtype = 1     '1 indicates fixed window length.  2 indicates fixed start point,   3 indicates fixed end point.
%varyingcoefs = %coefnames   'coefficients to vary through roll
%windowtypes = """Fixed Length Window"" ""Anchored at Start"" ""Anchored at End"""
!result = @uidialog("list",!windowtype,"Select a roll window type",%windowtypes, "edit", %varyingcoefs, "Enter the list of variables that can change", 256, "caption","Rolling specification")
if !result = -1 then
	stop
endif

%start = "@first"    'roll start point
%end = "@last"     'roll end point
%window = "20"    'window size
%step = "10"         'step size

'storage/display options
!StoreCoefs = 1      'store coefficients
!StoreSEs = 0          'store standard errors
!StorePs = 0             'store P-values
!StoreR2s = 0         'store R-squareds
!doGraphs = 1        'display graphs of the stored variables.

if !windowtype=1 then
	!result = @uidialog("edit",%start,"Start date/obs","edit",%end, "Last date/obs","edit",%window,"Window size","edit",%step,"Step size","colbreak","text","Storage options","check",!StoreCoefs,"Store Coefficients","check",!StoreSEs,"Store Std. Errors","check",!StorePs,"Store P-values","check",!StoreR2s,"Store R-squareds","check",!doGraphs,"Create Graphs","caption","Rolling Specification and Storage Options")
	if !result = -1 then
		stop
	endif
endif

if !windowtype=2 or !windowtype=3 then
	!result = @uidialog("edit",%start,"Start date/obs","edit",%end, "Last date/obs","edit",%step,"Step size","colbreak","text","Storage options","check",!StoreCoefs,"Store Coefficients","check",!StoreSEs,"Store Std. Errors","check",!StorePs,"Store P-values","check",!StoreR2s,"Store R-squareds","check",!doGraphs,"Create Graphs","caption","Rolling Specification and Storage Options")
	if !result = -1 then
		stop
	endif
endif

'check %window and %step are valid numbers
!temp = @val(%window )
if !temp = na or !temp <1 or !temp <> @floor(!temp ) then
	@uiprompt("Invalid window - please enter a positive integer.")
	stop
endif
!temp = @val(%step)
if !temp = na or !temp <1 or !temp <> @floor(!temp ) then
	@uiprompt("Invalid step - please enter a positive integer.")
	stop
endif


'set sample to all observerations covered
smpl {%start} {%end}
_this.{%estcmnd}  'initial full sample estimation of equation

'process non-varying regressors
%varyingcoefs = @upper(%varyingcoefs)
%nonvary = @wnotin(%coefnames,%varyingcoefs)
%cmnd = _this.@varlist
if @len(%nonvary) then
	%allcoefs = _this.@varlist  'this includes the dependent variable
	%dep = "(" + @word(%allcoefs,1)
	for !i=1 to @wcount(%nonvary)
		!idx = @wfind(%allcoefs,@word(%nonvary,!i))
		!coef = _this.@coefs(!idx-1)  'minus 1 since %allcoefs include dep variable.
		%dep = %dep + " -" + @str(!coef)+"*" + @word(%nonvary,!i)
	next
	%dep = %dep + ")"
	%cmnd = %dep + " " + %varyingcoefs
	%coefnames = %varyingcoefs
endif


if @len(_this.@instlist) then
	%cmnd = %cmnd + " @ " + _this.@instlist
endif
%estcmnd = _this.@method + "(" + _this.@options + ")" + " " + %cmnd

if !windowtype=1 then
	!window = @val(%window)
else 
	!window = 0
endif
!step = @val(%step)
!obs = @obssmpl
!rolls = (!obs-!window)/!step   'number of rolls that will take place

	
'Check whether number of rolls is an integer, warn other wise
if !rolls <> @floor(!rolls) then
	!rolls = @floor(!rolls)
	!end = @dtoo(%start) + (!rolls)*!step+!window-1
	%end = @otod(!end)
	%msg = "Warning: roll specification incompatible with start point and end point - adjusting end point to " + %end
	@uiprompt(%msg)
endif


'convert coefficient names to valid eviews names (if necessary)
for !i=1 to @wcount(%coefnames)
	%tempname = @word(%coefnames,!i)
	if @isvalidname(%tempname)=0 then
		%tempname = @makevalidname(%tempname)
	endif
	%tempstr = %tempstr + " " + %tempname	
next
%tempstr = @trim(%tempstr)
%coefnames = %tempstr

!NCoefs = @wcount(%coefnames)
'Declare series to store coefs
if !StoreCoefs = 1 then
	%cgrpname = "rollcoefs"
	if @isobject(%cgrpname) then
		%cgrpname = @getnextname(%cgrpname)
	endif
	group {%cgrpname}
	for !i=1 to !NCoefs
		%cname = @word(%coefnames,!i)
		%sername = %cgrpname + "_" + %cname
		series {%sername}
		{%sername}.displayname {%cname}
		{%cgrpname}.add {%sername}
	next
endif
'Declare series to store SEs
if !StoreSes = 1 then
	%sgrpname = "rollses"
	if @isobject(%sgrpname) then
		%sgrpname = @getnextname(%sgrpname)
	endif
	group {%sgrpname}
	for !i=1 to !NCoefs
		%cname = @word(%coefnames,!i)
		%sername = %sgrpname+ "_" + @word(%coefnames,!i)
		series {%sername}
		{%sername}.displayname {%cname}
		{%sgrpname}.add {%sername}
	next
endif
'Declare series to store PVals
if !StorePs = 1 then
	%pgrpname = "rollpvals"
	if @isobject(%pgrpname) then
		%pgrpname= @getnextname(%pgrpname)
	endif
	group {%pgrpname}
	for !i=1 to !NCoefs
		%cname = @word(%coefnames,!i)
		%sername = %pgrpname+ "_" + @word(%coefnames,!i)
		series {%sername}
		{%sername}.displayname {%cname}
		{%pgrpname}.add {%sername}
	next
endif
'Declare series to store R2s
if !StoreR2s = 1 then
	%rsername = "rollr2s"
	if @isobject(%rgrpname) then
		%rgrpname = @getnextname(%rgrpname)
	endif
	series {%rsername }
endif

if !windowtype=1 then
	'roll the sample
	for !i=1 to !obs-!window+1-!step step !step
		!k = !i+!window-1
		%first = @otod(@dtoo(%start)+!i-1)
		%last = @otod(@dtoo(%start)+!i+!window-2)
		smpl {%first} {%last}
		!kk = @dtoo(%start) + !k-1
	
		'estimate equation - note I'm error catching here, any errors mean we'll keep going.
		!maxerr = @maxerrcount
		setmaxerrs !maxerr+1
		_this.{%estcmnd}

		if @lasterrnum>0 then
			clearerrs
		else
			'store stuff
			if !StoreCoefs=1 then
				for !j=1 to !NCoefs
					%sername = {%cgrpname}.@seriesname(!j)
					{%sername}(!kk) = _this.@coef(!j)
				next
			endif
			if !StoreSEs=1 then
				for !j=1 to !NCoefs
					%sername = {%sgrpname}.@seriesname(!j)
					{%sername}(!kk) = _this.@stderrs(!j)
				next
			endif
			if !StorePs=1 then
				for !j=1 to !NCoefs
					!df = (_this.@regobs-_this.@ncoef)
					vector tempvec =@tdist(_this.@tstats,!df)
					%sername = {%pgrpname}.@seriesname(!j)
					{%sername}(!kk) = tempvec(!j)
					d tempvec
				next
			endif
			if !Storer2s=1 then
				{%rsername}(!kk)=_this.@r2
			endif
		endif
		setmaxerrs !maxerr
	next
endif

if !windowtype=2 or !windowtype=3 then
	for !i=1 to !obs+1-!step step !step

		if !windowtype=2 then
			%last = @otod(@dtoo(%start)+!i-1)
			smpl {%start} {%last}
			!k = @dtoo(%start)+!i-1
		else
			%first= @otod(@dtoo(%start)+!i-1)
			smpl {%first} {%end}
			!k = @dtoo(%start)+!i-1
		endif

		if @obssmpl> !ncoefs+1 then  'only estimate equation if there are enough observations.
			'estimate equation - note I'm error catching here, any errors mean we'll keep going.
			!maxerr = @maxerrcount
			setmaxerrs !maxerr+1
			_this.{%estcmnd}

			if @lasterrnum>0 then
				clearerrs
			else
				'store stuff
				if !StoreCoefs=1 then
					for !j=1 to !NCoefs
						%sername = {%cgrpname}.@seriesname(!j)
						{%sername}(!k) = _this.@coef(!j)
					next
				endif
				if !StoreSEs=1 then
					for !j=1 to !NCoefs
						%sername = {%sgrpname}.@seriesname(!j)
						{%sername}(!k) = _this.@stderrs(!j)
					next
				endif
				if !StorePs=1 then
					for !j=1 to !NCoefs
						!df = (_this.@regobs-_this.@ncoef)
						vector tempvec =@tdist(_this.@tstats,!df)
						%sername = {%pgrpname}.@seriesname(!j)
						{%sername}(!k) = tempvec(!j)
						d tempvec
					next
				endif
				if !Storer2s=1 then
					{%rsername}(!k)=_this.@r2
				endif
			endif
		endif
	next
endif	

'reset the sample
smpl {%origsmpl}
'put equation back to original state.
_this.{%origcmnd}

smpl @all
'make graphs
if !doGraphs=1 then
	if !StoreCoefs=1 then
		%cgraphname = "coefgraph"
		If @isobject(%cgraphname) then
			%cgraphname = @getnextname(%cgraphname)
		endif
		freeze({%cgraphname}) {%cgrpname}.line(m)
		for !i=1 to !NCoefs
			%varname = @word(%coefnames,!i) 
			{%cgraphname}.name(!i) {%varname}
		next
		{%cgraphname}.addtext(t, font(18)) "Rolling Coefficients"
		{%cgraphname}.options connect
		_this.display {%cgraphname}
	endif
	if !StoreSEs=1 then
		%sgraphname = "SEgraph"
		If @isobject(%sgraphname) then
			%sgraphname = @getnextname(%sgraphname)
		endif
		freeze({%sgraphname}) {%sgrpname}.line(m)
		for !i=1 to !NCoefs
			%varname = @word(%coefnames,!i) 
			{%sgraphname}.name(!i) {%varname}
		next
		{%sgraphname}.addtext(t, font(18)) "Rolling Standard Errors"
		{%sgraphname}.options connect
	endif
	if !StorePs=1 then
		%pgraphname = "PValgraph"
		If @isobject(%pgraphname) then
			%pgraphname = @getnextname(%pgraphname)
		endif
		freeze({%pgraphname}) {%pgrpname}.line(m)
		for !i=1 to !NCoefs
			%varname = @word(%coefnames,!i) 
			{%pgraphname}.name(!i) {%varname}
		next
		{%pgraphname}.addtext(t, font(18)) "Rolling P-Values"
		{%pgraphname}.options connect
	endif
	if !StoreR2s=1 then
		%rgraphname = "R2graph"
		If @isobject(%rgraphname) then
			%rgraphname = @getnextname(%rgraphname)
		endif
		freeze({%rgraphname}) {%rsername}.line()
		{%rgraphname}.addtext(t, font(18)) "Rolling R-Squares"
		{%rgraphname}.options connect
	endif
endif

'reset the sample
smpl {%origsmpl}


